// hlavickove subory *******************************************************
// systemove funkcie
#include <stdio.h>
#include <math.h>
// GL a GLU kniznica
//#include <GL/gl.h>
//#include <GL/glu.h>

// .h subor
#include "m3Dmath.h"

// vypocet velkosti vektora ************************************************
float m3Dvector::Magnitude(void)
{
 return(sqrt(x*x+y*y+z*z));
}

// normalizacia vektora ****************************************************
void m3Dvector::Normalize(void)
{
 M=Magnitude();

 if (M!=0.0f) { x/=M; y/=M; z/=M; }
}

// vzdialenost dvoch bodov *************************************************
float m3Dmath::Distance(m3Dvector pA,m3Dvector pB)
{
 return(sqrt((pB.x-pA.x)*(pB.x-pA.x)+(pB.y-pA.y)*(pB.y-pA.y)+(pB.z-pA.z)*(pB.z-pA.z)));
}

// skalarny sucin dvoch vektorov *******************************************
float m3Dmath::Dot(m3Dvector vector1,m3Dvector vector2)
{
 return((vector1.x*vector2.x)+(vector1.y*vector2.y)+(vector1.z*vector2.z));
}

// vektorovy sucin dvoch vektorov ******************************************
void m3Dmath::Cross(m3Dvector vector1,m3Dvector vector2,m3Dvector *vector3)
{
 vector3->x=vector1.y*vector2.z-vector1.z*vector2.y;
 vector3->y=vector1.z*vector2.x-vector1.x*vector2.z;
 vector3->z=vector1.x*vector2.y-vector1.y*vector2.x;
}

// vektorovy sucin dvoch vektorov ******************************************
m3Dvector m3Dmath::Cross2(m3Dvector vector1,m3Dvector vector2)
{
 return m3Dvector(vector1.y*vector2.z-vector1.z*vector2.y,
	              vector1.z*vector2.x-vector1.x*vector2.z,
	              vector1.x*vector2.y-vector1.y*vector2.x);
}

// vypocet normaly *********************************************************
void m3Dmath::CalcNormal(float p1,float p2,float p3,
                         float q1,float q2,float q3,
                         float r1,float r2,float r3,
                         float *a,float *b,float *c)
{
 v11=q1-p1; v12=q2-p2; v13=q3-p3; // vypocet prvkov 2 vektorov
 v21=r1-p1; v22=r2-p2; v23=r3-p3;

 n1=v12*v23-v13*v22;              // vektorovy sucin 2 vektorov
 n2=v13*v21-v11*v23;
 n3=v11*v22-v12*v21;

 // normalizacia vektora
 NormaliseNormal(n1,n2,n3,&nn1,&nn2,&nn3);
 // vysledok
 *a=nn1; *b=nn2; *c=nn3;
}

// normalizacia normaly ****************************************************
void m3Dmath::NormaliseNormal(float n1,float n2,float n3,
                              float *a,float *b,float *c)
{
 // ziskanie velkosti vektora
 d=sqrt(n1*n1+n2*n2+n3*n3);
 // uprava prvkov vektora, vysledok
 if (d!=0.0f) { *a=n1/d; *b=n2/d; *c=n3/d; }
 else { *a=0.0f; *b=0.0f; *c=0.0f; }
}

// urcenie polohy bodu voci rovine *****************************************
PointPos m3Dmath::ClassifyPoint(m3Dvector normal,float d,m3Dvector point)
{
 dist1=normal.x*point.x+normal.y*point.y+normal.z*point.z+d;

 if (dist1>0.0f) return POINTPOS_FRONT;
 else if (dist1<0.0f) return POINTPOS_BEHIND;
	  else return POINTPOS_ON;
}

// vypocet priesecnika hrany trojuholnika a roviny tvorenej trojuholnikom **
int m3Dmath::EdgePlaneIntersection(m3Dvector p1,m3Dvector p2,m3Dvector plane_normal,float d,m3Dvector *IP)
{
 side1=ClassifyPoint(plane_normal,d,p1);
 side2=ClassifyPoint(plane_normal,d,p2);

 if ((side1==side2)||(side1==POINTPOS_ON)||(side2==POINTPOS_ON)) return(0);
 
 // vypocet prvkov vektora P1->P2
 vec1=p2-p1;
 vec1.Normalize();
 // ziskanie vzdialenosti bodu P1 od roviny
 Numerator=-(plane_normal.x*p1.x+
             plane_normal.y*p1.y+
             plane_normal.z*p1.z+
             d);
 // skalarny sucin (dot product) dvoch vektorov
 Denominator=Dot(plane_normal,vec1);

 // v pripade, ze vektor P1->P2 je kolmy na normalu trojuholnika, odovzda sa bod P1
 if (Denominator==0.0f) *IP=p1;

 D=Numerator/Denominator;

 // priesecnik vektora P1->P2 a roviny tvorenej trojuholnikom
 IP->x=(p1.x+(vec1.x*D));
 IP->y=(p1.y+(vec1.y*D));
 IP->z=(p1.z+(vec1.z*D));

 return(1);
}

// vypocet novych texturovacich suradnic v priesecnikoch *******************
void m3Dmath::ComputeTextureCoords(m3Dvector A,m3Dvector B,m3Dvector C,
                                   m3Dvector At,m3Dvector Bt,m3Dvector Ct,
                                   unsigned int a1,
                                   unsigned int a2,
                                   m3Dvector p1,m3Dvector p2,
                                   m3Dvector *p1t,m3Dvector *p2t)
{
 switch (a1) {
  case 1: // p1t pretina hranu A->B
   dist1=Distance(A,B); dist2=Distance(A,p1);
   vec1=Bt-At;

   Pf=dist2/dist1;
   p1t->x=At.x+vec1.x*Pf; p1t->y=At.y+vec1.y*Pf;
   break;
  case 2: // p1t pretina hranu B->C
   dist1=Distance(B,C); dist2=Distance(B,p1);
   vec1=Ct-Bt;

   Pf=dist2/dist1;
   p1t->x=Bt.x+vec1.x*Pf; p1t->y=Bt.y+vec1.y*Pf;
   break;
  case 3: // p1t pretina hranu C->A
   dist1=Distance(C,A); dist2=Distance(C,p1);
   vec1=At-Ct;

   Pf=dist2/dist1;
   p1t->x=Ct.x+vec1.x*Pf; p1t->y=Ct.y+vec1.y*Pf;
   break;
 }

 switch (a2) {
  case 1: // p2t pretina hranu A->B
   dist1=Distance(A,B); dist2=Distance(A,p2);
   vec1=Bt-At;

   Pf=dist2/dist1;
   p2t->x=At.x+vec1.x*Pf; p2t->y=At.y+vec1.y*Pf;
   break;
  case 2: // p2t pretina hranu B->C
   dist1=Distance(B,C); dist2=Distance(B,p2);
   vec1=Ct-Bt;

   Pf=dist2/dist1;
   p2t->x=Bt.x+vec1.x*Pf; p2t->y=Bt.y+vec1.y*Pf;
   break;
  case 3: // p2t pretina hranu C->A
   dist1=Distance(C,A); dist2=Distance(C,p2);
   vec1=At-Ct;

   Pf=dist2/dist1;
   p2t->x=Ct.x+vec1.x*Pf; p2t->y=Ct.y+vec1.y*Pf;
   break;
 }
 
 // INFO: povodne a nove texturovacie suradnice
 //fprintf(stderr,"%f %f %f %f %f %f\n",At.x,At.y,Bt.x,Bt.y,Ct.x,Ct.y);
 //fprintf(stderr,"%f %f %f %f\n",p1t->x,p1t->y,p2t->x,p2t->y);
}

// test kolizie gule s trojuholnikom ***************************************
bool m3Dmath::IntersectedSphere(m3Dvector triangle[],
                                m3Dvector trianglenormal,
                                m3Dvector center,float radius,
                                float *dist)
{
 // vypocet vzdialenosti trojuholnika od pociatku suradnicovj sustavy
 // vypocitava sa D=-(Ax+By+Cz)
 // kde A,B,C=normala trojuholnika
 //     x,y,z=lubovolny vrchol tohto trojuholnika
 (*dist)=TriangleDistance(trianglenormal,triangle[0]);

 // vypocet vzdialenosti stredu gule od trojuholnika (roviny, ktoru tvori)
 distance1=trianglenormal.x*center.x+ // Ax+
           trianglenormal.y*center.y+ // By+
           trianglenormal.z*center.z+ // Cz+
           (*dist);                   // D

 // INFO: vzdielnost stredu gule od roviny
 //fprintf(stderr,"distance1: %f\n",distance1);

 // kolizia iba s privratenymi trojuholnikmi, ostatne sa neberu do uvahy
 // scena musi byt vytvorena tak, aby nemohlo dojst k prechodu cez odvrateny
 // trojuholnik
 if (distance1<0) return false;

 // ak je vzdialenost gule od roviny vacsia ako jej polomer, nedoslo ku kolizii
 if (myAbs(distance1)>=radius) return false;

 // presun stredu gule do roviny, ktoru tvori trojuholnik
 m_point=center-(trianglenormal*distance1);

 // gula kolidovala s trojuholnikom (rovinou, ktoru tvori)
 return true;
}

// test ci priesecnik patri trojuholniku, ak nie, nedoslo ku kolizii *******
bool m3Dmath::PointInTriangle(m3Dvector IPoint,m3Dvector triangleplanes[],float D1,float D2,float D3)
{
 // ak bude priesecnik aspon za jednou stranou, nie je v trojuholniku
 if (IPoint.x*triangleplanes[0].x+IPoint.y*triangleplanes[0].y+IPoint.z*triangleplanes[0].z+D1>0.0f) return false;
 if (IPoint.x*triangleplanes[1].x+IPoint.y*triangleplanes[1].y+IPoint.z*triangleplanes[1].z+D2>0.0f) return false;
 if (IPoint.x*triangleplanes[2].x+IPoint.y*triangleplanes[2].y+IPoint.z*triangleplanes[2].z+D3>0.0f) return false;

 // priesecnik je pred vsetkymi troma stranami => je vnutri v trojuholniku
 return true;
}

// test kolizie gule s hranami trojuholnika ********************************
bool m3Dmath::EdgeCollision(m3Dvector center,m3Dvector triangle[],float radius)
{
 // 1. hrana
 m_point=PointOnLine(triangle[0],triangle[1],center);
 if (Distance(m_point,center)<radius) return true;
 // 2. hrana
 m_point=PointOnLine(triangle[1],triangle[2],center);
 if (Distance(m_point,center)<radius) return true;
 // 3. hrana
 m_point=PointOnLine(triangle[2],triangle[0],center);
 if (Distance(m_point,center)<radius) return true;
 // ku kolizii nedoslo
 return false;
}

// vrati bod na ciare pA-pB, ktory je najblizsie k bodu pC *****************
m3Dvector m3Dmath::PointOnLine(m3Dvector pA,m3Dvector pB,m3Dvector pC)
{
 m_vectA=pC-pA;
 m_vectB=Normalize(pB-pA);

 D=Distance(pA,pB);

 // projekcia vektora m_vect1 do vektora m_vect2
 // ziskanie vzdialenosti projektovaneho vektoru od bodu pA
 M=Dot(m_vectB,m_vectA);

 // vzdialenost je mensia ako pA
 if (M<=0) return pA;

 // vzdialenost je vacsia ako pB
 if (M>=D) return pB;

 // vypocet vektora dlzky M v smere vektora m_vect2
 m_vector=m_vectB*M;

 // posun bodu pA pomocou vektora m_vect2
 return(pA+m_vector);
}

// normalizacia vektora ****************************************************
m3Dvector m3Dmath::Normalize(m3Dvector vector)
{
 M=sqrt((vector.x*vector.x)+(vector.y*vector.y)+(vector.z*vector.z));

 if (M!=0.0f) { vector.x/=M; vector.y/=M; vector.z/=M; }

 return vector;
}

// vypocet vzdialenosti trojuholnika od pociatku suradnicovej sustavy ******
float m3Dmath::TriangleDistance(m3Dvector normal,m3Dvector point)
{
 // pouzije sa rovnica roviny (Ax+By+Cz+D=0), podla ktorej sa vypocita D=-(Ax+By+Cz)
 return -((normal.x*point.x)+(normal.y*point.y)+(normal.z*point.z));
}

// vrati absolutnu hodnotu cisla *******************************************
float m3Dmath::myAbs(float N)
{
 if (N<0.0f) return (0.0f-N);

 return N;
}
 
// test kolizie kamery s trojuholnikom - s vyuzitim gule *******************
bool m3Dmath::CameraInTriangleS(m3Dvector triangle[],m3Dvector trianglenormal,m3Dvector center,float radius1,float radius2,
                                m3Dvector triangleplanes[],float D1,float D2, float D3)
{
 if (trianglenormal.y<=0.0f) radius1=radius2;

 // test kolizie gule s trojuholnikom, zaroven sa vypocitaju suradnice kolizneho bodu
 if (IntersectedSphere(triangle,trianglenormal,center,radius1,&distance0)) {
  if (PointInTriangle(m_point,triangleplanes,D1,D2,D3)) {
   // posun kamery v opacnom smere (efekt klzania po koliznej ploche)
   m_vector=trianglenormal*radius1;
   ret_pos=m_point+m_vector;
   // kamera kolidovala s trojuholnikom
   return true;
  }

  if (trianglenormal.y>0.0f) radius1=radius2;

  if (EdgeCollision(center,triangle,radius1)) {
   // posun kamery v opacnom smere (efekt klzania po koliznej hrane)
   m_vector=Normalize(center-m_point)*radius1;
   ret_pos=(m_point+m_vector);
   // kamera kolidovala s hranou trojuholnika
   return true;
  }
 }

 return false;
}

// vypocet vzdialenosti bodu od roviny *************************************
float m3Dmath::PointPlaneDistance(m3Dvector Normal,float D,m3Dvector Point)
{
 // pouzije sa rovnica roviny (Ax+By+Cz+D=0)
 dist1=Normal.x*Point.x+ // Ax+
       Normal.y*Point.y+ // By+
       Normal.z*Point.z+ // Cz+
       D;                // D

 return(dist1);
}
