#ifndef _M3DMATH_H_
#define _M3DMATH_H_

#define PI05 1.570796327f
#define PI1  3.141592653f
#define PI2	 6.283185306f
#define SQRT2DIV2 0.707106781f
#define GRAVITY 9.81f

enum PointPos // poloha bodu
{
 POINTPOS_FRONT=1,   // bod lezi pred rovinou
 POINTPOS_BEHIND=-1, // bod lezi za rovinou
 POINTPOS_ON=0       // bod lezi v rovine
};

enum TriPos // poloha trojuholnika
{
 TRIPOS_FRONT,   // trojuholnik lezi cely pred rovinou
 TRIPOS_BEHIND,  // trojuholnik lezi cely za rovinou
 TRIPOS_ON,      // trojuholnik lezi v rovine (je s nou rovnobezny)
 TRIPOS_SPANNING // trojuholnik lezi na oboch stranach roviny (pretina rovinu)
};

// trieda pre pracu s 3D bodmi a vektormi
class m3Dvector {
public:
 // konstruktor
 m3Dvector() {}
 // konstruktor s inicializaciou dat
 m3Dvector(float a,float b,float c) { x=a; y=b; z=c; }
 // nastavenie prvkov vektora
 inline void set(const float a,const float b,const float c);

 // pretazeny operator +
 m3Dvector operator+(m3Dvector mv) { return m3Dvector(mv.x+x,mv.y+y,mv.z+z); }
 // pretazeny operator -
 m3Dvector operator-(m3Dvector mv) { return m3Dvector(x-mv.x,y-mv.y,z-mv.z); }
 // pretazeny operator *
 m3Dvector operator*(float num) { return m3Dvector(x*num,y*num,z*num); }
 // prvky vektora (poloha 3D bodu)
 float x,y,z;

 // velkost vektora
 float M;
 // vypocet velkosti vektora
 float Magnitude(void);
 // normalizacia vektora
 void Normalize(void);
};

// nastavenie prvkov vektora
inline void m3Dvector::set(const float a,const float b,const float c) { x=a; y=b; z=c; }

// trieda matematicko-geometrickych vypoctov
class m3Dmath {
public:
 // konstruktor
 m3Dmath() {}

 // vzdialenost dvoch bodov
 float Distance(m3Dvector pA,m3Dvector pB);
	 
 // skalarny sucin dvoch vektorov
 float Dot(m3Dvector vector1,m3Dvector vector2);

 // vektorovy sucin dvoch vektorov
 void Cross(m3Dvector vector1,m3Dvector vector2,m3Dvector *vector3);
 // vektorovy sucin dvoch vektorov
 m3Dvector Cross2(m3Dvector vector1,m3Dvector vector2);

 // vypocet normaly
 void CalcNormal(float p1,float p2,float p3,
                 float q1,float q2,float q3,
                 float r1,float r2,float r3,
                 float *a,float *b,float *c);

 // normalizacia normaly
 void NormaliseNormal(float n1,float n2,float n3,
                      float *a,float *b,float *c);

 // urcenie polohy bodu voci rovine	 
 PointPos ClassifyPoint(m3Dvector normal,float d,m3Dvector point);

 // vypocet priesecnika hrany trojuholnika a roviny tvorenej trojuholnikom
 int EdgePlaneIntersection(m3Dvector p1,m3Dvector p2,m3Dvector plane_normal,float d,m3Dvector *IP);

 // vypocet novych texturovacich suradnic v priesecnikoch
 void ComputeTextureCoords(m3Dvector A,m3Dvector B,m3Dvector C,
	                       m3Dvector At,m3Dvector Bt,m3Dvector Ct,
                           unsigned int a1,
                           unsigned int a2,
                           m3Dvector p1,m3Dvector p2,
                           m3Dvector *p1t,m3Dvector *p2t);

 // test kolizie kamery s trojuholnikom - s vyuzitim gule
 bool CameraInTriangleS(m3Dvector triangle[],m3Dvector trianglenormal,m3Dvector center,float radius1,float radius2,
                        m3Dvector triangleplanes[],float D1,float D2, float D3);

 // nova poloha kamery ovplyvnena koliziou s objektom sceny
 m3Dvector ret_pos;
 
 // vypocet vzdialenosti bodu od roviny
 float PointPlaneDistance(m3Dvector Normal,float D,m3Dvector Point);
private:
 // nasledovne premenne su pouzivane pri vypoctoch normal
 float v11,v12,v13;          // 1. vektor
 float v21,v22,v23;          // 2. vektor
 float n1,n2,n3,nn1,nn2,nn3; // pomocne premenne
 float d;                    // velkost vektora

 PointPos side1,side2; // urcenie polohy bodu

 float dist1,dist2,dist3,dist4; // vypocet vzdialenosti

 m3Dvector vec1,vec2; // pomocne vektory

 float Numerator,Denominator,D,M; // pre vypocet priesecnika bodu s rovinou

 float Pf; // vypocet pomeru vzdialenosti

 m3Dvector m_vectA,m_vectB;
 m3Dvector m_vector,m_point; // pre docasne ulozenie vektora a bodu

 float distance0,distance1,distance2; // pre vypocet vzdialenosti

 // test kolizie gule s trojuholnikom
 bool IntersectedSphere(m3Dvector triangle[],
                        m3Dvector trianglenormal,
                        m3Dvector center,float radius,
                        float *dist);

 // test ci priesecnik patri trojuholniku, ak nie, nedoslo ku kolizii
 bool PointInTriangle(m3Dvector IPoint,m3Dvector triangleplanes[],float D1,float D2,float D3);

 // test kolizie gule s hranami trojuholnika
 bool EdgeCollision(m3Dvector center,m3Dvector triangle[],float radius);

 // vrati bod na ciare pA-pB, ktory je najblizsie k bodu pC
 m3Dvector PointOnLine(m3Dvector pA,m3Dvector pB,m3Dvector pC);
 
 // normalizacia vektora
 m3Dvector Normalize(m3Dvector vector);
 
 // vypocet vzdialenosti trojuholnika od pociatku suradnicovej sustavy
 float TriangleDistance(m3Dvector normal,m3Dvector point);
 
 // vrati absolutnu hodnotu cisla
 float myAbs(float N);
};
 
#endif
