#ifndef _MCAMERA_H_
#define _MCAMERA_H_

// matematicke operacie pre vypocet kolizii
#include "m3Dmath.h"

// trieda pre pracu s OpenGL kamerou
class mCamera {
public:
 mCamera();          // konstruktor
 virtual ~mCamera(); // destruktor

 // reset parametrov kamery
 void cam_reset(void);
 // reset casu
 void cam_resettime(void);
 // reinicializacia kamery
 void cam_reinit(GLfloat w,GLfloat h);
 // nastavenie perspektivneho premietania
 void cam_SetPerspective(void);
 // rotacia kamery
 void cam_rotate_xy(GLfloat x,GLfloat y);
 void cam_rotate_z(GLfloat z);
 // posun kamery
 void cam_move(void);
 // aktualizacia nastaveni kamery
 void cam_update(void);
 // zap/vyp detekcie kolizii
 void CollisionOnOff(void);

 GLfloat scr_w,scr_h; // sirka, vyska okna

 GLfloat matrix[16];                             // matica kamery
 GLfloat posx,posy,posz;                         // poloha
 GLfloat oldx,oldy,oldz;                         // povodna poloha
 GLfloat targetx,targety,targetz;                // smer
 
 // indikatory pre vykonavanie definovanych posunov kamery
 GLfloat goForward,goBack,goUp,goDown,goRight,goLeft;
 GLubyte goJump,goFast;

 GLubyte isFire1; // strelba1 (lave tlacidlo mysi)
 GLubyte isFire2; // strelba2 (prave tlacidlo mysi)

 GLubyte movetype; // typ pohybu

 GLfloat AVATAR_eye;  // vyska oci AVATARa - potrebne pre vypocet kolizii
 GLfloat AVATAR_wide; // sirka AVATARa - potrebne pre vypocet kolizii

 GLfloat ogl_fps; // pocitadlo FPS (snimky za sekundu)

 // detekcia kolizie kamery s objektom sceny (trojuholnikom)
 int cam_collision(GLfloat x1,GLfloat y1,GLfloat z1,
	               GLfloat x2,GLfloat y2,GLfloat z2,
	               GLfloat x3,GLfloat y3,GLfloat z3,
                   GLfloat tr_nx,GLfloat tr_ny,GLfloat tr_nz,
                   GLfloat tr_cn1,GLfloat tr_cn2, GLfloat tr_cn3, GLfloat tr_cn4,
                   GLfloat tr_cn5,GLfloat tr_cn6, GLfloat tr_cn7, GLfloat tr_cn8,
                   GLfloat tr_cn9,GLfloat tr_cn10,GLfloat tr_cn11,GLfloat tr_cn12);	  
 // indikator zap/vyp dektekcie kolizii
 GLubyte collision_onoff;
 // indikator vyskutu kolizie 
 GLubyte is_collision;
 
 // cas medzi vykreslenim 2 snimkov
 GLfloat GetCameraTime(void);
private:
 GLuint t_a,t_b; // pomocne premenne

 GLfloat movx,movy,movz;                // posun kamery
 GLfloat angx,angy,angz,rotx,roty,rotz; // otocenie, rotacia
 GLfloat speed,slide,alt;               // rychlost, bocny pohyb, zmena vysky
 GLfloat fspeed,fslide,falt;            // pre rychlejsi pohyb
 GLfloat acceleration,gravity;          // zrychlenie, gravitacne zrychlenie
 GLfloat viewnear,viewfar;              // minimalny a maximalny dohlad
 GLfloat FOV;                           // uhol pohladu
 GLfloat JumpPower;                     // sila skoku

 GLfloat movx_factor,movy_factor,movz_factor; // konstanty pre upravu rychlosti posunu
 GLfloat rotx_factor,roty_factor,rotz_factor; // konstanty pre upravu rychlosti rotacie
 
 // ulozenie stareho a noveho casoveho okamihu (ms)
 GLuint actTime,oldTime;
 // cas vykreslenia sceny
 GLfloat difTime;
 // cas posobenia gravitacie
 GLfloat gravityTime;
 // pocitadlo pre FPS (pocet snimkov)
 GLfloat ogl_frames;
};

#endif
