// hlavickove subory *******************************************************
// systemove funkcie
#include <stdlib.h>
#include <math.h>
// GL a GLU kniznica
#include <GL/gl.h>
#include <GL/glu.h>
// funkcie SDL kniznice
#include <SDL/SDL.h>

#include "m3Dmath.h"
// pre vektorove vypocty
m3Dmath m_math;

#include "mTexture.h"
// pre texturu kamenov
mTexture m_texture;

// .h subor
#include "mWater.h"

// konstruktor *************************************************************
mWater::mWater()
{
 v=NULL;  // vrcholy siete
 vN=NULL; // normaly vrcholov
 vF=NULL; // "sila" posobiaca na vrchol
 vV=NULL; // rychlost vrcholu

 w_freq=1.0f;       // frekvencia
 w_wavelength=1.0f; // dlzka_vlny
 w_muffle=1.0f;     // utlm

 rock_id=0; // prvy kamen
 for (i=0;i<20;i++) {
  rock[i].y=-100.0f;   // na zaciatku su vsetky kamene "pod zemou"
  rock[i].lifeT=-1.0f; // reset casu zivota kamenov
  rock[i].dropT=0;     // reset dopadu kamena na hladinu vody
 }
}

// destruktor **************************************************************
mWater::~mWater()
{
 // uvolnenie alokovanej pamate
 WaterDestroy();
}

// inicializacia vody ******************************************************
int mWater::WaterInit(float SX,float SZ,int DX,int DZ)
{
 WaterDestroy(); // uvolnenie alokovanej pamate

 sx=SX; sz=SZ; // rozmer siete
 dx=DX; dz=DZ; // pocet deleni rozmeru siete

 v=new m3Dvector[dx*dz];  // alokovanie pamate pre vrcholy siete
 if (v==NULL) {
  fprintf(stderr,"Chyba pri alokovani miesta pre vrcholy siete!\n"); return(1);
 }
 vN=new m3Dvector[dx*dz]; // alokovanie pamate pre normaly vrcholov
 if (vN==NULL) {
  fprintf(stderr,"Chyba pri alokovani miesta pre normaly vrcholov!\n"); return(1);
 }

 vF=new float[dx*dz]; // "sila" posobiaca na vrchol
 vV=new float[dx*dz]; // rychlost vrcholu

 WaterReset(); // reset parametrov vody

 // nahratie textury kamenov
 if (m_texture.LoadTexture(0,"data/material/rock.tga",TEXTURE_CLAMP)) {
  fprintf(stderr,"Chyba nacitania textury kamenov!\n"); return(1);	 
 }

 return(0);
}

// reset parametrov vody ***************************************************
void mWater::WaterReset(void)
{
 for (z=0;z<dz;z++)	{
  zz=z/(float) (dz-1); zz=(zz-0.5f)*sz;
  for (x=0;x<dx;x++) {
   xx=x/(float) (dx-1); xx=(xx-0.5f)*sx;

   v[z*dx+x].set(xx,0.0f,zz);      // nastavenie pociatocnych hodnot vrcholov siete
   vN[z*dx+x].set(0.0f,1.0f,0.0f); // nastavenie pociatocnych hodnot normal vrcholov
  }
 }

 // nastavenie pociatocnych hodnot "sily" a rychlosti
 for (i=0;i<dx*dz;i++) { vF[i]=0.0f; vV[i]=0.0f; }
}

// vykreslenie vody ********************************************************
void mWater::WaterRender(int type)
{
 switch (type) {
  case 0: // vyplnena siet
   glPushAttrib(GL_TEXTURE_BIT);
   glDisable(GL_TEXTURE_2D); // vypnutie texturovania

   // zapnutie priesvitnosti
   glBlendFunc(GL_ONE,GL_ONE_MINUS_SRC_ALPHA);
   glEnable(GL_BLEND);

   // farba vody
   glColor4f(97.0f/256.0f,102.0f/256.0f,143.0f/256.0f,0.5f);
   for (z=0;z<dz-1;z++)	{
	glBegin(GL_TRIANGLES);
	for (x=0;x<dx-1;x++) {
	 glNormal3f(vN[z*dx+x].x,vN[z*dx+x].y,vN[z*dx+x].z);
	 glVertex3f(v[z*dx+x].x,v[z*dx+x].y,v[z*dx+x].z);
	 glNormal3f(vN[(z+1)*dx+x].x,vN[(z+1)*dx+x].y,vN[(z+1)*dx+x].z);
	 glVertex3f(v[(z+1)*dx+x].x,v[(z+1)*dx+x].y,v[(z+1)*dx+x].z);
	 glNormal3f(vN[z*dx+(x+1)].x,vN[z*dx+(x+1)].y,vN[z*dx+(x+1)].z);
	 glVertex3f(v[z*dx+(x+1)].x,v[z*dx+(x+1)].y,v[z*dx+(x+1)].z);

     glNormal3f(vN[z*dx+(x+1)].x,vN[z*dx+(x+1)].y,vN[z*dx+(x+1)].z);
	 glVertex3f(v[z*dx+(x+1)].x,v[z*dx+(x+1)].y,v[z*dx+(x+1)].z);
	 glNormal3f(vN[(z+1)*dx+x].x,vN[(z+1)*dx+x].y,vN[(z+1)*dx+x].z);
	 glVertex3f(v[(z+1)*dx+x].x,v[(z+1)*dx+x].y,v[(z+1)*dx+x].z);
	 glNormal3f(vN[(z+1)*dx+(x+1)].x,vN[(z+1)*dx+(x+1)].y,vN[(z+1)*dx+(x+1)].z);
	 glVertex3f(v[(z+1)*dx+(x+1)].x,v[(z+1)*dx+(x+1)].y,v[(z+1)*dx+(x+1)].z);
	}
	glEnd();
   }
   glColor4f(1.0f,1.0f,1.0f,1.0f); // default-biela farba
 
   // vypnutie priesvitnosti
   glDisable(GL_BLEND);

   glPopAttrib();
  break;
  case 1: // jednotlive vrcholy siete
   glPushAttrib(GL_LIGHTING_BIT|GL_TEXTURE_BIT);
   glDisable(GL_LIGHTING);   // vypnutie osvetlenia
   glDisable(GL_TEXTURE_2D); // vypnutie texturovania

   glColor4f(0.0f,0.0f,1.0f,1.0f);
   glBegin(GL_POINTS);
   for (z=0;z<dz;z++) {
	for (x=0;x<dx;x++) {
	 glNormal3f(vN[z*dx+x].x,vN[z*dx+x].y,vN[z*dx+x].z);
	 glVertex3f(v[z*dx+x].x,v[z*dx+x].y,v[z*dx+x].z);
	}
   }
   glEnd();
   glColor4f(1.0f,1.0f,1.0f,1.0f); // default-biela farba

   glPopAttrib();
  break;
  case 2: // ciary siete
   glPushAttrib(GL_LIGHTING_BIT|GL_TEXTURE_BIT);
   glDisable(GL_LIGHTING);   // vypnutie osvetlenia
   glDisable(GL_TEXTURE_2D); // vypnutie texturovania

   glColor4f(0.0f,0.0f,1.0f,1.0f);
   for (z=0;z<dz;z++) {
    glBegin(GL_LINE_STRIP);
	for (x=0;x<dx;x++) {
	 glNormal3f(vN[z*dx+x].x,vN[z*dx+x].y,vN[z*dx+x].z);
	 glVertex3f(v[z*dx+x].x,v[z*dx+x].y,v[z*dx+x].z);
	}
	glEnd();
   }
   for (x=0;x<dx;x++) {
	glBegin(GL_LINE_STRIP);
    for (z=0;z<dz;z++) {
	 glNormal3f(vN[z*dx+x].x,vN[z*dx+x].y,vN[z*dx+x].z);
	 glVertex3f(v[z*dx+x].x,v[z*dx+x].y,v[z*dx+x].z);
	}
	glEnd();
   }
   glColor4f(1.0f,1.0f,1.0f,1.0f); // default-biela farba

   glPopAttrib();
  break;
  default: break;
 }
}

// vykreslenie normal ******************************************************
void mWater::WaterRenderNormals(float size)
{
 glPushAttrib(GL_LIGHTING_BIT|GL_TEXTURE_BIT);
 glDisable(GL_LIGHTING);   // vypnutie osvetlenia
 glDisable(GL_TEXTURE_2D); // vypnutie texturovania

 glColor4f(1.0f,0.0f,0.0f,1.0f); // cervena farba
 glBegin(GL_LINES);
 for (z=0;z<dz;z++) {
  for (x=0;x<dx;x++) {
   glVertex3f(v[z*dx+x].x,v[z*dx+x].y,v[z*dx+x].z);
   glVertex3f(v[z*dx+x].x+size*(vN[z*dx+x].x),
	          v[z*dx+x].y+size*(vN[z*dx+x].y),
              v[z*dx+x].z+size*(vN[z*dx+x].z));
  }
 }
 glEnd();
 glColor4f(1.0f,1.0f,1.0f,1.0f); // default-biela farba

 glPopAttrib();
}

// simulacia padu objektu do vody ******************************************
void mWater::WaterDrop(float DPx,float DPy,float DPz)
{
 // miesto dopadu objektu na povrch hladiny vody
 drop_point.x=DPx;
 drop_point.y=DPy;
 drop_point.z=DPz;

 // zmena Y-ovej suradnice vsetkych vrcholov siete
 for (z=0;z<dz;z++)	{
  for (x=0;x<dx;x++) {
   // vypocet vzdialenosti vrcholu od miesta dopadu
   D=XZDistance(drop_point,v[z*dx+x]);
   // vypocet novej Y-ovej suradnice vrcholov siete
   //
   // pouzije sa vzorec: Y+=miesto_dopadu.Y*cos(D*2*PI/dlzka_vlny)*exp(-D*D)
   //
   v[z*dx+x].y+=drop_point.y*(float) cos(D*PI2/w_wavelength)*(float) exp(-D*D);
   // vypocet Y-ovej suradnice vrcholov zavisly na case	  
   //v[z*dx+x].y+=drop_point.y*(float) cos(PI2*w_freq-D*PI2/w_wavelength)*(float) exp(-D*D*w_muffle);
  }
 }
}

// casovo zavisla zmena stavu hladiny vody *********************************
void mWater::WaterUpdate(float chTime)
{
 // reset "sily" posobiacej na vrcholy
 for (i=0;i<dx*dz;i++) vF[i]=0.0f;
 // pouzije sa okolie vrcholu pozostavajuce z 8-mich okolitych vrcholov:
 //  0 1 0
 //  1 V 1
 //  0 1 0
 // vypocet rozdielu vysky stredneho a okrajoveho vrcholu:
 //  vyska stredneho vrcholu sa umerne znizi
 //  vyska okrajoveho vrcholu sa umerne zvysi
 // plati pravidlo:
 //  okolite vrcholy okrem vrcholov leziacich na diagonale maju vahu 1.0f
 //  vrcholy leziace na diagonale maju vahu 0.707f
 // stredny vrchol:
 //  F8=-2*(4*1.0f+4*0.707f)=-13.656f
 //  V8+=2.828f*F8/13.656f - hranica stability pre 8-okolie
 //  F4=-2*(4*1.0f)=-8.0f
 //  V4+=2*F4/8.0f - hranica stability pre 4-okolie
 for (z=1;z<dz-1;z++) {
  for (x=1;x<dx-1;x++) {
   // stredny vrchol
   i=dx*z+x;
   // 8 okolitych vrcholov
   j=dx*(z  )+x+1; D=1.0f*(v[i].y-v[j].y);      vF[i]-=D; vF[j]+=D; // pravy
   j=dx*(z  )+x-1; D=1.0f*(v[i].y-v[j].y);      vF[i]-=D; vF[j]+=D; // lavy
   j=dx*(z-1)+x  ; D=1.0f*(v[i].y-v[j].y);      vF[i]-=D; vF[j]+=D; // horny
   j=dx*(z+1)+x  ; D=1.0f*(v[i].y-v[j].y);      vF[i]-=D; vF[j]+=D; // dolny
   j=dx*(z-1)+x+1; D=SQRT2DIV2*(v[i].y-v[j].y); vF[i]-=D; vF[j]+=D; // pravy horny
   j=dx*(z-1)+x-1; D=SQRT2DIV2*(v[i].y-v[j].y); vF[i]-=D; vF[j]+=D; // lavy horny
   j=dx*(z+1)+x+1; D=SQRT2DIV2*(v[i].y-v[j].y); vF[i]-=D; vF[j]+=D; // pravy dolny
   j=dx*(z+1)+x-1; D=SQRT2DIV2*(v[i].y-v[j].y); vF[i]-=D; vF[j]+=D; // lavy dolny
  }
 }

 // vypocet rychlosti zavisly od FPS
 chVfps=30.0f*chTime; if (chVfps>1.0f) chVfps=1.0f;
 // vypocet utlmu zavisly od FPS
 chMfps=(float) exp(250.0f*chTime*(float) log(1.0f-0.01f*w_muffle));
 
 // INFO: rychlost_fps
 //fprintf(stderr,"%f\n",chVfps);
 
 for(i=0;i<dx*dz;i++) {
  // zvacsenie rychlosti vrcholu:
  //  v+=rychlost_fps*sila*tekutost_kvapaliny
  //  vysledok nesmie byt vacsi ako hranica stability
  //  V8+=rychlost_fps*F8*0.207088459f
  //  pokial pouzijeme 0.2f tak rychlost_fps moze byt max. 1.035442295f
  //  v nasom pripade je obmedzena na 1.0f
  vV[i]+=chVfps*vF[i]*0.2f;
  // zmena vysky vrcholu podla rychlosti vrcholu:
  //  y+=rychlost_fps*rychlost
  //  y*=utlm
  v[i].y+=chVfps*vV[i];
  v[i].y*=chMfps;
 }

 // vypocet novych hodnot normal
 WaterNormals();
}

// vypocet normal pocas zmeny stavu hladiny vody ***************************
void mWater::WaterNormals(void)
{
 // rychly, ale nepresny vypocet, normala vrcholu
 // sa vypocita na zaklade laveho trojuholnika z dvojice,
 // ktora spolu vytvrara jednotlive stvorce siete
 /*
 for (z=0;z<dz-1;z++) {
  for (x=0;x<dx-1;x++) {
   m_math.CalcNormal(v[z*dx+x].x,v[z*dx+x].y,v[z*dx+x].z,
	                 v[(z+1)*dx+x].x,v[(z+1)*dx+x].y,v[(z+1)*dx+x].z,
	                 v[z*dx+(x+1)].x,v[z*dx+(x+1)].y,v[z*dx+(x+1)].z,
	                 &xx,&yy,&zz);
   vN[z*dx+x].x=xx; vN[z*dx+x].y=yy; vN[z*dx+x].z=zz;
  }
 }
 */
 // pomaly, ale presny vypocet, normala vrcholu
 // sa vypocita na zaklade priemeru normal 6-tich susediacich
 // trojuholnikov	
 for (z=1;z<dz-1;z++) {
  for (x=1;x<dx-1;x++) {
   // 1. trojuholnik
   m_math.CalcNormal(v[(z-1)*dx+x].x,v[(z-1)*dx+x].y,v[(z-1)*dx+x].z,
                     v[z*dx+(x-1)].x,v[z*dx+(x-1)].y,v[z*dx+(x-1)].z,
                     v[z*dx+x].x,v[z*dx+x].y,v[z*dx+x].z,
                     &xx,&yy,&zz);
   vN[z*dx+x].x=xx; vN[z*dx+x].y=yy; vN[z*dx+x].z=zz;
   // 2. trojuholnik
   m_math.CalcNormal(v[z*dx+(x-1)].x,v[z*dx+(x-1)].y,v[z*dx+(x-1)].z,
                     v[(z+1)*dx+(x-1)].x,v[(z+1)*dx+(x-1)].y,v[(z+1)*dx+(x-1)].z,
                     v[z*dx+x].x,v[z*dx+x].y,v[z*dx+x].z,
                     &xx,&yy,&zz);
   vN[z*dx+x].x+=xx; vN[z*dx+x].y+=yy; vN[z*dx+x].z+=zz;
   // 3. trojuholnik
   m_math.CalcNormal(v[z*dx+x].x,v[z*dx+x].y,v[z*dx+x].z,
	                 v[(z+1)*dx+(x-1)].x,v[(z+1)*dx+(x-1)].y,v[(z+1)*dx+(x-1)].z,
	                 v[(z+1)*dx+x].x,v[(z+1)*dx+x].y,v[(z+1)*dx+x].z,
                     &xx,&yy,&zz);
   vN[z*dx+x].x+=xx; vN[z*dx+x].y+=yy; vN[z*dx+x].z+=zz;
   // 4. trojuholnik
   m_math.CalcNormal(v[z*dx+x].x,v[z*dx+x].y,v[z*dx+x].z,
	                 v[(z+1)*dx+x].x,v[(z+1)*dx+x].y,v[(z+1)*dx+x].z,
                     v[z*dx+(x+1)].x,v[z*dx+(x+1)].y,v[z*dx+(x+1)].z,
                     &xx,&yy,&zz);
   vN[z*dx+x].x+=xx; vN[z*dx+x].y+=yy; vN[z*dx+x].z+=zz;
   // 5. trojuholnik
   m_math.CalcNormal(v[(z-1)*dx+(x+1)].x,v[(z-1)*dx+(x+1)].y,v[(z-1)*dx+(x+1)].z,
                     v[z*dx+x].x,v[z*dx+x].y,v[z*dx+x].z,
					 v[z*dx+(x+1)].x,v[z*dx+(x+1)].y,v[z*dx+(x+1)].z,
                     &xx,&yy,&zz);
   vN[z*dx+x].x+=xx; vN[z*dx+x].y+=yy; vN[z*dx+x].z+=zz;
   // 6. trojuholnik
   m_math.CalcNormal(v[(z-1)*dx+x].x,v[(z-1)*dx+x].y,v[(z-1)*dx+x].z,
                     v[z*dx+x].x,v[z*dx+x].y,v[z*dx+x].z,
                     v[(z-1)*dx+(x+1)].x,v[(z-1)*dx+(x+1)].y,v[(z-1)*dx+(x+1)].z,
                     &xx,&yy,&zz);
   vN[z*dx+x].x+=xx; vN[z*dx+x].y+=yy; vN[z*dx+x].z+=zz;
   // priemer spocitanych hodnot
   vN[z*dx+x].x/=6.0f; vN[z*dx+x].y/=6.0f; vN[z*dx+x].z/=6.0f;
  }
 }
}

// XZ-vzdialenost dvoch vrcholov siete *************************************
float mWater::XZDistance(const m3Dvector VA,const m3Dvector VB)
{
 xx=VA.x-VB.x; zz=VA.z-VB.z;
 return (float) sqrt(xx*xx+zz*zz);
}

// uvolnenie alokovanej pamate *********************************************
void mWater::WaterDestroy(void)
{
 if (v!=NULL) delete v; v=NULL;    // vrcholy siete
 if (vN!=NULL) delete vN; vN=NULL; // normaly vrcholov
 if (vF!=NULL) delete vF; vF=NULL; // "sila" posobiaca na vrchol
 if (vV!=NULL) delete vV; vV=NULL; // rychlost vrcholu
}

// hodenie kamena **********************************************************
void mWater::RockThrow(float Cx,float Cy,float Cz,float Tx,float Ty,float Tz)
{
 // aktivnych je vzdy iba 20 kamenov, v pripade, ze je hodeny 21. kamen,
 // prepisu sa parametre 1. kamena...
 if (rock_id>=20) rock_id=0;

 // prva stvorica vrcholov (lave)
 xx=(float) rand()/((float) RAND_MAX*4.0f);
 rock[rock_id].vert[0]=-1.0f-xx; rock[rock_id].vert[1]=-0.5f-xx; rock[rock_id].vert[2]= -0.5f-xx;
 xx=(float) rand()/((float) RAND_MAX*4.0f);
 rock[rock_id].vert[3]=-1.0f-xx; rock[rock_id].vert[4]=-0.5f-xx; rock[rock_id].vert[5]=  0.5f+xx;
 xx=(float) rand()/((float) RAND_MAX*4.0f);
 rock[rock_id].vert[6]=-1.0f-xx; rock[rock_id].vert[7]= 0.5f+xx; rock[rock_id].vert[8]=  0.5f+xx;
 xx=(float) rand()/((float) RAND_MAX*4.0f);
 rock[rock_id].vert[9]=-1.0f-xx; rock[rock_id].vert[10]=0.5f+xx; rock[rock_id].vert[11]=-0.5f-xx;

 // druha stvorica vrcholov (center)
 xx=(float) rand()/((float) RAND_MAX*2.0f);
 rock[rock_id].vert[12]=0.0f+xx; rock[rock_id].vert[13]=-1.0f-xx; rock[rock_id].vert[14]=-1.0f-xx;
 xx=(float) rand()/((float) RAND_MAX*2.0f);
 rock[rock_id].vert[15]=0.0f+xx; rock[rock_id].vert[16]=-1.0f-xx; rock[rock_id].vert[17]= 1.0f+xx;
 xx=(float) rand()/((float) RAND_MAX*2.0f);
 rock[rock_id].vert[18]=0.0f+xx; rock[rock_id].vert[19]= 1.0f+xx; rock[rock_id].vert[20]= 1.0f+xx;
 xx=(float) rand()/((float) RAND_MAX*2.0f);
 rock[rock_id].vert[21]=0.0f+xx; rock[rock_id].vert[22]= 1.0f+xx; rock[rock_id].vert[23]=-1.0f-xx;

 // tretia stvorica vrcholov (prave)
 xx=(float) rand()/((float) RAND_MAX*4.0f);
 rock[rock_id].vert[24]=1.0f+xx; rock[rock_id].vert[25]=-0.5f-xx; rock[rock_id].vert[26]=-0.5f-xx;
 xx=(float) rand()/((float) RAND_MAX*4.0f);
 rock[rock_id].vert[27]=1.0f+xx; rock[rock_id].vert[28]=-0.5f-xx; rock[rock_id].vert[29]= 0.5f+xx;
 xx=(float) rand()/((float) RAND_MAX*4.0f);
 rock[rock_id].vert[30]=1.0f+xx; rock[rock_id].vert[31]= 0.5f+xx; rock[rock_id].vert[32]= 0.5f+xx;
 xx=(float) rand()/((float) RAND_MAX*4.0f);
 rock[rock_id].vert[33]=1.0f+xx; rock[rock_id].vert[34]= 0.5f+xx; rock[rock_id].vert[35]=-0.5f-xx;
 
 // velkost kamena
 xx=(float) rand()/((float) RAND_MAX*4.0f);
 //fprintf(stderr,"%f\n",xx);
 rock[rock_id].s=1.0f*xx;

 // poloha kamena
 rock[rock_id].x=Cx; rock[rock_id].y=Cy; rock[rock_id].z=Cz;
 // uhly, pod ktorymi bol kamen hodeny (prvky smeroveho vektora)
 rock[rock_id].a=Tx; rock[rock_id].b=Ty; rock[rock_id].c=Tz;

 // INFO: uhly, pod ktorymi bol kamen hodeny 
 //fprintf(stderr,"%3.2f %3.2f %3.2f\n",rock[rock_id].a,rock[rock_id].b,rock[rock_id].c);

 // rychlost kamena
 rock[rock_id].v=2.0f;
 
 // cas "zivota" kamena
 rock[rock_id].lifeT=0.0f;
 // identifikator dopadu na vodnu hladinu
 rock[rock_id].dropT=0;

 // zvysenie poctu kamenov
 rock_id++;
}

// vyvoj "zivota" hodenych kamenov *****************************************
void mWater::RocksUpdate(float chTime)
{
 for (i=0;i<20;i++) {
  if (rock[i].lifeT!=-1.0f) { // ak bol kamen hodeny
   // zvysenie casu "zivota" kamena
   rock[i].lifeT+=chTime;

   // zmena polohy kamena
   // kamen kopiruje trajektoriu urcenu nasledovnymi vztahmi:
   //  x=x0+v0*a*t
   //  y=y0+v0*b*t-g*t*t
   //  z=z0+v0*c*t
   //
   //  x,y,z - aktualna poloha kamena
   //  x0,y0,z0 - startovacia poloha kamena
   //  v0 - rychlost kamena
   //  a,b,c - smerovy vektor, ktorym bol kamen hodeny
   //          tento vektor nahradil povodne funkcie sin() a cos()
   //  t - cas "zivota" kamena
   //  g - gravitacne zrychlenie
   rock[i].x+=rock[i].v*rock[i].a*rock[i].lifeT;              // X

   g_quad=GRAVITY*rock[i].lifeT*rock[i].lifeT;
   rock[i].y+=rock[i].v*rock[i].b*rock[i].lifeT-g_quad/10.0f; // Y

   rock[i].z+=rock[i].v*rock[i].c*rock[i].lifeT;              // Z

   // dopad kamena na povrch hladiny vody
   if (rock[i].dropT==0) {
	// test, ci je to prvy dopad kamena
	if ((rock[i].y<=0.0f)&&(rock[i].y!=-2.0f)) {
	 rock[i].dropT=1;
	 // dopad kamena sposobi vlnenie hladiny vody
	 WaterDrop(rock[i].x,rock[i].y,rock[i].z);
	}
   }

   // pad kamena sa zastavi 2.0f jednotiek pod hladinou vody
   if (rock[i].y<=-2.0f) {
	rock[i].y=-2.0f;     // kamen sa zastavi 2.0f jednotiek pod hladinou vody
	rock[i].lifeT=-1.0f; // kamen uz nebude dalej padat
	rock[i].dropT=0;     // ak sa kamen pouzije znovu, moze sposobit vlnenie
   }
  }
 }
}

// vykreslenie kamenov *****************************************************
void mWater::RocksRender(void)
{
 // pripojenie textury kamenov
 glBindTexture(GL_TEXTURE_2D,m_texture.texture_id[0]);
 
 for (i=0;i<20;i++) {
  if (rock[i].y!=-100.0f) { // na zaciatku su kamene "pod zemou"
   // ak bol kamen hodeny, vykresli sa
   glPushMatrix();
   // poloha
   glTranslatef(rock[i].x,rock[i].y,rock[i].z);
   // velkost
   glScalef(rock[i].s,rock[i].s,rock[i].s);
/*
   // kocka
   glBegin(GL_QUADS);
	glTexCoord2f(0.0f,0.0f);
    glVertex3f(-1.0f,-1.0f,1.0f); // predna stena
	glTexCoord2f(1.0f,0.0f);
    glVertex3f( 1.0f,-1.0f,1.0f);
	glTexCoord2f(1.0f,1.0f);
	glVertex3f( 1.0f, 1.0f,1.0f);
	glTexCoord2f(0.0f,1.0f);
	glVertex3f(-1.0f, 1.0f,1.0f);

    glVertex3f( 1.0f,-1.0f,-1.0f); // zadna stena
    glVertex3f(-1.0f,-1.0f,-1.0f);
	glVertex3f(-1.0f, 1.0f,-1.0f);
	glVertex3f( 1.0f, 1.0f,-1.0f);

    glVertex3f( 1.0f,-1.0f, 1.0f); // prava stena
    glVertex3f( 1.0f,-1.0f,-1.0f);
	glVertex3f( 1.0f, 1.0f,-1.0f);
	glVertex3f( 1.0f, 1.0f, 1.0f);

    glVertex3f(-1.0f,-1.0f,-1.0f); // lava stena
    glVertex3f(-1.0f,-1.0f, 1.0f);
	glVertex3f(-1.0f, 1.0f, 1.0f);
	glVertex3f(-1.0f, 1.0f,-1.0f);

    glVertex3f(-1.0f,1.0f, 1.0f); // horna stena
    glVertex3f( 1.0f,1.0f, 1.0f);
	glVertex3f( 1.0f,1.0f,-1.0f);
	glVertex3f(-1.0f,1.0f,-1.0f);

    glVertex3f(-1.0f,-1.0f,-1.0f); // dolna stena
    glVertex3f( 1.0f,-1.0f,-1.0f);
	glVertex3f( 1.0f,-1.0f, 1.0f);
	glVertex3f(-1.0f,-1.0f, 1.0f);
   glEnd();
*/
   // 10-sten
   glBegin(GL_TRIANGLE_STRIP); // po obvode
    glTexCoord2f(0.0f/6.0f,2.0f/6.0f);
    glVertex3f(rock[i].vert[9],rock[i].vert[10],rock[i].vert[11]);
    glTexCoord2f(0.0f/6.0f,1.0f/6.0f);
    glVertex3f(rock[i].vert[0],rock[i].vert[1],rock[i].vert[2]);

    glTexCoord2f(1.0f/6.0f,2.0f/6.0f);
    glVertex3f(rock[i].vert[6],rock[i].vert[7],rock[i].vert[8]);
	glTexCoord2f(1.0f/6.0f,1.0f/6.0f);
    glVertex3f(rock[i].vert[3],rock[i].vert[4],rock[i].vert[5]);

    glTexCoord2f(2.0f/6.0f,2.0f/6.0f);
    glVertex3f(rock[i].vert[18],rock[i].vert[19],rock[i].vert[20]);
    glTexCoord2f(2.0f/6.0f,1.0f/6.0f);
    glVertex3f(rock[i].vert[15],rock[i].vert[16],rock[i].vert[17]);

    glTexCoord2f(3.0f/6.0f,2.0f/6.0f);
    glVertex3f(rock[i].vert[30],rock[i].vert[31],rock[i].vert[32]);
    glTexCoord2f(3.0f/6.0f,1.0f/6.0f);
    glVertex3f(rock[i].vert[27],rock[i].vert[28],rock[i].vert[29]);

    glTexCoord2f(4.0f/6.0f,2.0f/6.0f);
    glVertex3f(rock[i].vert[33],rock[i].vert[34],rock[i].vert[35]);
    glTexCoord2f(4.0f/6.0f,1.0f/6.0f);
    glVertex3f(rock[i].vert[24],rock[i].vert[25],rock[i].vert[26]);

    glTexCoord2f(5.0f/6.0f,2.0f/6.0f);
    glVertex3f(rock[i].vert[21],rock[i].vert[22],rock[i].vert[23]);
    glTexCoord2f(5.0f/6.0f,1.0f/6.0f);
    glVertex3f(rock[i].vert[12],rock[i].vert[13],rock[i].vert[14]);

    glTexCoord2f(6.0f/6.0f,2.0f/6.0f);
    glVertex3f(rock[i].vert[9],rock[i].vert[10],rock[i].vert[11]);
    glTexCoord2f(6.0f/6.0f,1.0f/6.0f);
    glVertex3f(rock[i].vert[0],rock[i].vert[1],rock[i].vert[2]);
   glEnd();
   glBegin(GL_TRIANGLE_STRIP); // vrcha strana
    glTexCoord2f(1.0f/6.0f,3.0f/6.0f);
    glVertex3f(rock[i].vert[9],rock[i].vert[10],rock[i].vert[11]);
    glTexCoord2f(1.0f/6.0f,2.0f/6.0f);
    glVertex3f(rock[i].vert[6],rock[i].vert[7],rock[i].vert[8]);

    glTexCoord2f(2.0f/6.0f,3.0f/6.0f);
    glVertex3f(rock[i].vert[21],rock[i].vert[22],rock[i].vert[23]);
    glTexCoord2f(2.0f/6.0f,2.0f/6.0f);
    glVertex3f(rock[i].vert[18],rock[i].vert[19],rock[i].vert[20]);

    glTexCoord2f(3.0f/6.0f,3.0f/6.0f);
    glVertex3f(rock[i].vert[33],rock[i].vert[34],rock[i].vert[35]);
    glTexCoord2f(3.0f/6.0f,2.0f/6.0f);
    glVertex3f(rock[i].vert[30],rock[i].vert[31],rock[i].vert[32]);
   glEnd(); 
   glBegin(GL_TRIANGLE_STRIP); // spodna strana
    glTexCoord2f(1.0f/6.0f,1.0f/6.0f);
    glVertex3f(rock[i].vert[3],rock[i].vert[4],rock[i].vert[5]);
    glTexCoord2f(1.0f/6.0f,0.0f/6.0f);
    glVertex3f(rock[i].vert[0],rock[i].vert[1],rock[i].vert[2]);

    glTexCoord2f(2.0f/6.0f,1.0f/6.0f);
    glVertex3f(rock[i].vert[15],rock[i].vert[16],rock[i].vert[17]);
    glTexCoord2f(2.0f/6.0f,0.0f/6.0f);
    glVertex3f(rock[i].vert[12],rock[i].vert[13],rock[i].vert[14]);

    glTexCoord2f(3.0f/6.0f,1.0f/6.0f);
    glVertex3f(rock[i].vert[27],rock[i].vert[28],rock[i].vert[29]);
    glTexCoord2f(3.0f/6.0f,0.0f/6.0f);
    glVertex3f(rock[i].vert[24],rock[i].vert[25],rock[i].vert[26]);
   glEnd();

   glPopMatrix();
  }
 }
}
